#include "module.h"

#ifndef NO_SCRIPT_ACTIONS

const char *ScriptActionStrings[ScriptActionModuleCount+1] =
{
  /* These MUST be in the same order as the */ 
  /* entries in the ScriptActionType enum   */ 

  /* module or window */ 
  "Create", "Destroy",
  "MouseOver", "MouseOut",
  "MouseMove", "Hover", "Unhover",
  "LButtonDown", "LButtonUp",
  "RButtonDown", "RButtonUp",
  "MButtonDown", "MButtonUp",

  /* module only */ 
  "Init", "Quit",

  NULL
};

int LookupVar(char *dest,int maxchars,const char *src,LPARAM lParam)
{
  LPARAM values[4];
  POINT pt;
  RECT r;
  DWORD d;
  bool bIntValue = false;
  bool left,right;
  int i;
  WndInfo *w = reinterpret_cast<WndInfo *>(lParam);

  switch (*src++)
  {
	case '_': /* module info */ 
		switch (*src++) {
    case 'N':
		case 'n': lstrcpyn(dest,ModuleName,maxchars); break;
    case 'V':
		case 'v': lstrcpyn(dest,ModuleVersion,maxchars); break;
    case 'A':
		case 'a': lstrcpyn(dest,ModuleAuthor,maxchars); break;
    case 'B':
    case 'b': lstrcpyn(dest,ModuleBuild,maxchars); break;
    default: return 0;
		}
		break;

  case '#': /* special char */ 
    switch (*src)
    {
    case 'A':
    case 'a': *dest = '\''; break;
    case 'C':
    case 'c': *dest = '^'; break;
    case 'D':
    case 'd': *dest = '$'; break;
    case 'N':
    case 'n': *dest = '\n'; break;
    case 'Q':
    case 'q': *dest = '"'; break;
    default:  *dest = *src;
    }
    return 1;

  case 'K':
	case 'k': /* keys */ 
		switch (*src++) {
    case 'A':
		case 'a': /* alt              */ 
			left = IsKeyDown(VK_LMENU);
			right = IsKeyDown(VK_RMENU);
			break;
    case 'C':
		case 'c': /* ctrl             */ 
			left = IsKeyDown(VK_LCONTROL);
			right = IsKeyDown(VK_RCONTROL);
			break;
    case 'M':
		case 'm': /* context menu key */ 
			left = right = IsKeyDown(VK_APPS);
			break;
    case 'S':
		case 's': /* shift            */ 
			left = IsKeyDown(VK_LSHIFT);
			right = IsKeyDown(VK_RSHIFT);
			break;
    case 'W':
		case 'w': /* win key          */ 
			left = IsKeyDown(VK_LWIN);
			right = IsKeyDown(VK_RWIN);
			break;
    default: return 0;
		}
		i = 3;
		switch (*src++) {
		case 'l': i=1; break;
		case 'r': i=2; break;
		}
		if ((i&1)==0) left = false;
		if ((i&2)!=0) left |= right;
		/* result */ 
		if (left) lstrcpyn(dest,"true",maxchars);
    else lstrcpyn(dest,"false",maxchars);
		break;

  case 'M':
  case 'm': /* mouse */ 
    d = GetMessagePos();
    pt.x = (short)LOWORD(d);
    pt.y = (short)HIWORD(d);
    switch (*src++)
    {
    case 'X':
    case 'x':
    case 'Y':
    case 'y':
      bIntValue = true;
      switch (*src)
      {
      case 's':
      case 'S':
        break;
      default:
        if (w) ScreenToClient(w->hWnd,&pt);
      }
      values[0] = (src[-1]=='x'||src[-1]=='X')?pt.x:pt.y;
      break;
    default: return 0;
    }
    break;

  case 'I':
  case 'i': /* window */ 
    /* return nothing if no window */ 
    if (!w) return 0;
    GetWindowRect(w->hWnd,&r);
    switch (*src++)
    {
    case 'X':
    case 'x':
      values[0] = r.left;
      bIntValue = true;
      break;
    case 'Y':
    case 'y':
      values[0] = r.top;
      bIntValue = true;
      break;
    case 'S':
    case 's': /* size */ 
      switch (*src++)
      {
      case 'X':
      case 'x': /* width */ 
        values[0] = r.right-r.left;
        bIntValue = true;
        break;
      case 'Y':
      case 'y': /* height */ 
        values[0] = r.bottom-r.top;
        bIntValue = true;
        break;
      default: return 0;
      }
      break;
    case 'H':
    case 'h': /* handle */ 
      values[0] = reinterpret_cast<LPARAM>(w->hWnd);
      bIntValue = true;
      break;
#ifndef SINGLE_WINDOW
    case 'N':
    case 'n':
      lstrcpyn(dest,w->Name,maxchars);
      break;
    case 'G':
    case 'g':
      values[0] = w->Group;
      bIntValue = true;
      break;
#endif
#ifndef NO_SKIN
    case 'B':
    case 'b':
      values[0] = reinterpret_cast<LPARAM>(w->hbmBackBuffer);
      bIntValue = true;
      break;
    case 'K':
    case 'k':
      values[0] = reinterpret_cast<LPARAM>(w->emSkin.Bitmap);
      bIntValue = true;
      break;
#endif
    default: return 0;
    }
    break;

  default: return 0;
  }

  if (bIntValue)
  {
    FormatMessage(
      FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
      "%1!d!",
      NULL,NULL,dest,maxchars,
      reinterpret_cast<char**>(values));
  }

  return lstrlen(dest);
}

void ExecActionString(HWND hCaller,WndInfo *w,const char *sAction)
{
  char buffer[MAX_LINE_LENGTH];
  if (sAction)
  {
    ReplaceAtVars(buffer,MAX_LINE_LENGTH,sAction,
      &LookupVar,reinterpret_cast<LPARAM>(w));
    LSExecute(hCaller,buffer,SW_SHOWNORMAL);
  }
}

void ExecAction(WndInfo *w,int iAction)
{
  const char *sAction = NULL;

  ASSERT(iAction<ScriptActionModuleCount);

#ifndef SINGLE_WINDOW
  if (w && iAction<ScriptActionWindowCount)
  {
    ASSERT(READABLE(w,sizeof(*w)));
    if (w->Actions)
    {
      ASSERT(READABLE(w->Actions,ScriptActionWindowCount*sizeof(ScriptAction)));
      sAction = w->Actions[iAction];
    }
  }

  /* only exec global action if no specific one */ 
  if (!sAction)
#endif

  if (Module.Actions)
  {
    ASSERT(READABLE(Module.Actions,ScriptActionModuleCount*sizeof(ScriptAction)));
    sAction = Module.Actions[iAction];
  }

  ExecActionString(w?w->hWnd:NULL,w,sAction);
}

#endif /* NO_SCRIPT_ACTIONS */ 