#include "module.h"

//
// Remember to update these for your module!
//

const char * const ModuleName     = "SkinBox";
const char * const ModuleVersion  = "1.0b2";
const char * const ModuleAuthor   = "RabidCow";

/* array so sizeof() works */ 
const char ModuleBangPrefix[]     = "!SkinBox";
const int ModuleBangPrefixLen     = sizeof(ModuleBangPrefix)-1;
const char ModuleRCPrefix[]       = "SkinBox";
const int ModuleRCPrefixLen       = sizeof(ModuleRCPrefix)-1;
#ifndef SINGLE_WINDOW
const char ModuleListKey[]        = "*SkinBox";
const int ModuleListKeyLen        = sizeof(ModuleListKey)-1;
#endif

//
//
//

#ifdef SINGLE_WINDOW
#define CONFIG_SW "Sngl "
#else
#define CONFIG_SW "Mlti "
#endif

#ifdef NO_SCRIPT_ACTIONS
#define CONFIG_SA "NoSc "
#else
#define CONFIG_SA "Scpt "
#endif

#ifdef USE_LSTOOLS
#define CONFIG_LS "LSTl "
#else
#define CONFIG_LS "StLS "
#endif

#ifdef NO_SKIN
#define CONFIG_SKIN "Whit "
#else
#define CONFIG_SKIN "Skin "
#endif

const char * const ModuleBuild = CONFIG_LS CONFIG_SW CONFIG_SA CONFIG_SKIN;

extern "C" {
__declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath);
__declspec( dllexport ) int quitModule(HINSTANCE dll);
#ifdef SINGLE_WINDOW
__declspec( dllexport ) int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType *wd);
__declspec( dllexport ) int quitWharfModule(HINSTANCE dllInst);
#endif
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

ModInfo Module;

BOOL  (WINAPI *dAlphaBlend)( IN HDC, IN int, IN int, IN int, IN int,
    IN HDC, IN int, IN int, IN int, IN int, IN BLENDFUNCTION) = NULL;
BOOL (WINAPI *dUpdateLayeredWindow)(HWND,HDC,POINT*,SIZE*,HDC,POINT*,
		COLORREF,BLENDFUNCTION*,DWORD) = NULL;

#define LINKLIB(name) { name, NULL },
#define LINKFUNC(name,ptr) { name, reinterpret_cast<int**>(&ptr) },

lsLinkItem DynFuncs[] =
{
	LINKFUNC("UpdateLayeredWindow",dUpdateLayeredWindow)

	LINKLIB("MSIMG32.DLL")
	LINKFUNC("AlphaBlend",dAlphaBlend)

	/* end of list marker */ 
	{ NULL, NULL }
};

// Initialize Module
//
//  ParentWnd = litestep or wharf window
//  dll       = instance handle for this module
//  szPath    = path to litestep.exe
//
// return 0 on success (failure is more or less ignored)
//
int initModuleEx(HWND ParentWnd, HINSTANCE dll, LPCSTR szPath)
{
	Config c;

  /* start with a clean slate */ 
	ZeroMemory(&Module,sizeof(ModInfo));
	Module.hInstance = dll;
  /* load optionally linked functions */ 
	LSLinkFunctions(DynFuncs);

  /* register window class */ 
	WNDCLASS wc;
	ZeroMemory(&wc,sizeof(WNDCLASS));
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.lpfnWndProc = &WndProc;
	wc.hInstance = dll;
	wc.lpszClassName = ModuleName;
	wc.cbWndExtra = sizeof(WndInfo *);
	if (!RegisterClass(&wc)) 
	{
		MessageBox(ParentWnd,
			"Error registering window class.\n\nPerhaps the module did not unload properly?",
			ModuleName,MB_OK|MB_ICONERROR);
		return 1;
	}

  /* find the appropriate parent for "always on bottom" and such */ 
	if (ParentWnd == GetLitestepWnd())
		/* using the litestep window as a parent will get me dropped behind  */ 
		/* the desktop if i'm in "always on bottom" mode, so use the desktop */ 
		Module.hBottomParentWnd = NULL;
	else
		Module.hBottomParentWnd = ParentWnd;
	if (!Module.hBottomParentWnd)
		Module.hBottomParentWnd = FindWindow("DesktopBackgroundClass", NULL);
	if (!Module.hBottomParentWnd)
		Module.hBottomParentWnd = GetDesktopWindow();

#ifdef SINGLE_WINDOW
	Module.MainWnd.hParentWnd = Module.hBottomParentWnd;
#endif

#ifndef USE_LSTOOLS
  LSInitMultiMon();
#endif

  /* read module-wide configuration */ 
	ZeroMemory(&c,sizeof(Config));
  c.tokens[0] = c.one;
  c.tokens[1] = c.two;
  c.tokens[2] = c.three;
  c.tokens[3] = c.four;
	ReadConfigMod(&Module,&c,ModuleRCPrefix,sizeof(ModuleRCPrefix)-1);

#ifdef SINGLE_WINDOW
  /* read config for window and create it */ 
	if (!Module.bWaitForBox)
	{
		ReadConfigWnd(&Module.MainWnd,&c,ModuleRCPrefix,sizeof(ModuleRCPrefix)-1);
		if (!CreateWnd(&Module.MainWnd)) return 1;
	}
#else
  /* read predefined windows */ 
  LSParseStarList(NULL,ModuleListKey,ParseBoxProc,
    &Module.pWnds,sizeof(WndInfo),(DWORD)&c);
  CreateWindows();
#endif

  /* register bang commands */ 
	LSRegisterBangList(ModuleBangPrefix,sizeof(ModuleBangPrefix)-1,Bangs);

#ifndef NO_SCRIPT_ACTIONS
  ExecAction(NULL,OnInit);
#endif

	return 0;
}

// Quit Module
//
//  dll = instance handle for module
//
// return 0 on success (failure is more or less ignored)
//
int quitModule(HINSTANCE dll)
{
#ifndef NO_SCRIPT_ACTIONS
  ExecAction(NULL,OnQuit);
#endif

  /* unregister bang commands */ 
	LSRemoveBangList(ModuleBangPrefix,sizeof(ModuleBangPrefix)-1,Bangs);
#ifdef SINGLE_WINDOW
  /* destroy the main window */ 
	DestroyWindow(Module.MainWnd.hWnd);
#else
  LSFreeStarList(FreeWndProc,Module.pWnds,NULL);
#endif

#ifndef USE_LSTOOLS
  LSQuitMultiMon();
#endif

  /* unregister window class */ 
	UnregisterClass(ModuleName,Module.hInstance);
  /* unload any dlls we loaded for optional funcs */ 
	LSUnlinkFunctions(DynFuncs);

	return 0;
}

#ifdef SINGLE_WINDOW
// hooks for loading as a wharf module
// just pass the params to regular init/quit functions
int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType *wd)
{ return initModuleEx (ParentWnd, dllInst, (wd) ? wd->lsPath : NULL); }
int quitWharfModule(HINSTANCE dllInst)
{ return quitModule(dllInst); }
#endif

