#include "module.h"

// create a window and mark it as part of litestep
bool CreateWnd(WndInfo *w)
{
	const static UINT LiteStepMessages[] = {
		LM_GETREVID,
		0
	};
	int x,y;
	x = w->Pos.x; y = w->Pos.y;
	/* convert the coords unless we are parented */ 
	if (w->hParentWnd==Module.hBottomParentWnd)
		LSConvertCoords(w->Monitor,w->bOnTop,&x,&y);
	if (w->bOnTop && w->hParentWnd==Module.hBottomParentWnd)
		w->hWnd =
		CreateWindowEx(
			WS_EX_TOOLWINDOW|WS_EX_TOPMOST|(w->bTrueAlpha?WS_EX_LAYERED:0),
			ModuleName,
			NULL,
			WS_POPUP,
			x, y,
			w->Size.cx, w->Size.cy,
			NULL, /* no parent for on top windows */ 
			NULL,
			Module.hInstance,
			NULL);
	else
		w->hWnd =
		CreateWindowEx(
			WS_EX_TOOLWINDOW|WS_EX_TOPMOST|(w->bTrueAlpha?WS_EX_LAYERED:0),
			ModuleName,
			NULL,
			WS_CHILD,
			x, y,
			w->Size.cx, w->Size.cy,
			w->hParentWnd,
			NULL,
			Module.hInstance,
			NULL);

	if (!w->hWnd) return false;

	SetWindowLong(w->hWnd,UGWL_WNDINFO_PTR,(LONG)w);
	SetWindowLong(w->hWnd,GWL_USERDATA,magicDWord);

#ifndef SINGLE_WINDOW
  /* only show up once in revid list */ 
  if (w->pNext == NULL)
#endif
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE,
		(WPARAM) w->hWnd, (LPARAM) LiteStepMessages);

  UpdateWndHidden(w);

#ifndef NO_SCRIPT_ACTIONS
  ExecAction(w,OnCreate);
#endif

	return true;
}

// update size and position of window
void UpdateWndPos(WndInfo *w)
{
	POINT p;
	p.x = w->Pos.x; p.y = w->Pos.y;
	/* convert the coords unless we are parented */ 
	ASSERT(sizeof(int)==sizeof(long));
	if (w->hParentWnd==Module.hBottomParentWnd)
		LSConvertCoords(w->Monitor,w->bOnTop,
      reinterpret_cast<int *>(&p.x),
      reinterpret_cast<int *>(&p.y));
	if (w->bTrueAlpha && dUpdateLayeredWindow!=NULL)
		dUpdateLayeredWindow(w->hWnd,NULL,&p,&w->Size,NULL,NULL,0,NULL,0);
	else
		SetWindowPos(w->hWnd,NULL,p.x,p.y,w->Size.cx,w->Size.cy,SWP_NOZORDER|SWP_NOACTIVATE);

	DeleteObject(w->hbmBackBuffer);
	w->hbmBackBuffer = NULL;

	RedrawWnd(w);
}

// update on top/bottem setting
void UpdateWndOnTop(WndInfo *w)
{
  /* Remove WS_POPUP and make it a WS_CHILD */ 
	SetWindowLong(w->hWnd, GWL_STYLE,
		(GetWindowLong(w->hWnd, GWL_STYLE) &~ WS_POPUP)|WS_CHILD);
	if (w->hParentWnd != Module.hBottomParentWnd) {
		/* in box, set parent to box window                              */ 
		SetParent(w->hWnd, w->hParentWnd);
		/* and leave as WS_CHILD (ignore bOnTop)                         */ 
	} else {
		/* need to reset position because ConvertCoords depends on ontop */ 
		UpdateWndPos(w);
		/* Set the Parent window to Desktop for "onBottom" emulation.    */ 
		/* Or set the Parent to NULL if "onTop" is desired               */ 
		SetParent(w->hWnd, w->bOnTop ? NULL : w->hParentWnd);
		/* Set the Style back to WS_POPUP so things function correctly.  */ 
		SetWindowLong(w->hWnd, GWL_STYLE,
		 (GetWindowLong(w->hWnd, GWL_STYLE) &~ WS_CHILD)|WS_POPUP);
	}
}

// update hide/show
void UpdateWndHidden(WndInfo *w)
{
	ShowWindow(w->hWnd,w->bHidden?SW_HIDE:SW_SHOWNA);
}

#ifndef SINGLE_WINDOW
void CreateWindows(void)
{
	WndInfo *w;

	w = Module.pWnds;
	while (w)
	{
		ASSERT(READABLE(w,sizeof(WndInfo)));
		CreateWnd(w);
		w = w->pNext;
	}
}

void AddWindow(WndInfo *w)
{
	WndInfo **p = &Module.pWnds;
	while (*p) p = &(*p)->pNext;
	*p = w;
	CreateWnd(w);
}
#endif /* SINGLE_WINDOW */ 